LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_signed.all;

ENTITY part2 IS
   PORT ( CLOCK_50, CLOCK2_50, AUD_DACLRCK   : IN    STD_LOGIC;
          AUD_ADCLRCK, AUD_BCLK, AUD_ADCDAT  : IN    STD_LOGIC;
          KEY                                : IN    STD_LOGIC_VECTOR(0 DOWNTO 0);
          FPGA_I2C_SDAT                      : INOUT STD_LOGIC;
			 SW											: IN STD_LOGIC_VECTOR(1 downto 0);
          FPGA_I2C_SCLK, AUD_DACDAT, AUD_XCK : OUT   STD_LOGIC
			 );
END part2;

ARCHITECTURE Behavior OF part2 IS
   COMPONENT clock_generator
      PORT( CLOCK2_50 : IN STD_LOGIC;
            reset    : IN STD_LOGIC;
            AUD_XCK  : OUT STD_LOGIC);
   END COMPONENT;

   COMPONENT audio_and_video_config
      PORT( CLOCK_50, reset : IN    STD_LOGIC;
            I2C_SDAT        : INOUT STD_LOGIC;
            I2C_SCLK        : OUT   STD_LOGIC);
   END COMPONENT;   

   COMPONENT audio_codec
      PORT( CLOCK_50, reset, read_s, write_s               : IN  STD_LOGIC;
            writedata_left, writedata_right                : IN  STD_LOGIC_VECTOR(23 DOWNTO 0);
            AUD_ADCDAT, AUD_BCLK, AUD_ADCLRCK, AUD_DACLRCK : IN  STD_LOGIC;
            read_ready, write_ready                        : OUT STD_LOGIC;
            readdata_left, readdata_right                  : OUT STD_LOGIC_VECTOR(23 DOWNTO 0);
            AUD_DACDAT                                     : OUT STD_LOGIC);
   END COMPONENT;
	
	function shr3( a : IN STD_LOGIC_VECTOR(23 DOWNTO 0)) return STD_LOGIC_VECTOR is
	begin
	   return (a(23) & a(23) & a(23) & a(23 DOWNTO 3));
	end function;
	

	CONSTANT Len : natural := 8;
	TYPE Buffer_Type is array (1 to Len-1) of STD_LOGIC_VECTOR(23 DOWNTO 0);
	

	procedure ShiftAndSum (buff : in Buffer_Type; 
								  dataIn : in STD_LOGIC_VECTOR(23 DOWNTO 0); 
								  signal dataOut : out STD_LOGIC_VECTOR(23 DOWNTO 0)) 
	is
		variable acc : STD_LOGIC_VECTOR(23 DOWNTO 0);
	begin
		acc := shr3(dataIn);
		for i in 1 to LEN-1 loop
			 acc := acc + shr3(buff(i));
		end loop;
		dataOut <= acc;
	end;
	
   SIGNAL read_ready, write_ready, read_s, write_s : STD_LOGIC;
   SIGNAL readdata_left, readdata_right            : STD_LOGIC_VECTOR(23 DOWNTO 0);
   SIGNAL writedata_left, writedata_right          : STD_LOGIC_VECTOR(23 DOWNTO 0);   
   SIGNAL reset                                    : STD_LOGIC;
	SIGNAL buffer_left, buffer_right 					: Buffer_Type;
	SIGNAL noise												: STD_LOGIC_VECTOR(23 DOWNTO 0);
	SIGNAL DataAndNoise_left, DATAAndNoise_right		: STD_LOGIC_VECTOR(23 DOWNTO 0);
	SIGNAL OutData_left, OutData_right					: STD_LOGIC_VECTOR(23 DOWNTO 0);
 
BEGIN
   reset <= NOT(KEY(0));

   --YOUR CODE GOES HERE
   --writedata_left <= readdata_left;
   --writedata_right <= readdata_right;
   read_s <= read_ready and write_ready;
   write_s <= read_ready and write_ready;
	
BufferData:	
	process(CLOCK_50, reset) is
		constant zero : STD_LOGIC_VECTOR(23 DOWNTO 0)  := (others=> '0');
	begin
		if reset='1' then
			buffer_left<= (others=> zero);
			buffer_right<= (others=> zero);
		elsif (rising_edge(CLOCK_50)) then
			if (read_s='1') then
				buffer_left(2 to LEN-1) <= buffer_left(1 to LEN-2);
				buffer_left(1)<=readdata_left;
				buffer_right(2 to LEN-1) <= buffer_right(1 to LEN-2);
				buffer_right(1)<=readdata_right;
			end if;
		end if;
	end process;
	
	DataAndNoise_left <= readdata_left+noise;
	DATAAndNoise_right<= readdata_right+noise;
	
SaS1:
   ShiftAndSum(buffer_left, DataAndNoise_left, OutData_left);
		
SaS2:
   ShiftAndSum(buffer_right, DATAAndNoise_right, OutData_right); 

NG: entity work.noise_generator(Behavior)
    port map( CLOCK_50, read_s, noise);	
   
   my_clock_gen: clock_generator PORT MAP (CLOCK2_50, reset, AUD_XCK);
   cfg: audio_and_video_config PORT MAP (CLOCK_50, reset, FPGA_I2C_SDAT, FPGA_I2C_SCLK);
   codec: audio_codec PORT MAP (CLOCK_50, reset, read_s, write_s, writedata_left, 
	                             writedata_right, AUD_ADCDAT, AUD_BCLK, AUD_ADCLRCK,
										  AUD_DACLRCK, read_ready, write_ready, readdata_left, 
										  readdata_right, AUD_DACDAT);
										  
	writedata_left <= OutData_left when SW="00" else 
							DataAndNoise_left when SW="01" else
							readdata_left when SW = "10" else
							6;
	writedata_right<= OutData_right when SW="00" else 
							DataAndNoise_right when SW="01" else
							readdata_right when SW = "10" else
							noise;
END Behavior;


	
