LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_signed.all;

ENTITY part3 IS
   PORT ( CLOCK_50, CLOCK2_50, AUD_DACLRCK   : IN    STD_LOGIC;
          AUD_ADCLRCK, AUD_BCLK, AUD_ADCDAT  : IN    STD_LOGIC;
          KEY                                : IN    STD_LOGIC_VECTOR(0 DOWNTO 0);
          FPGA_I2C_SDAT                      : INOUT STD_LOGIC;
			 SW											: IN STD_LOGIC_VECTOR(1 downto 0);
          FPGA_I2C_SCLK, AUD_DACDAT, AUD_XCK : OUT   STD_LOGIC
			 );
END part3;

ARCHITECTURE Behavior OF part3 IS
   CONSTANT K : natural := 5;
   CONSTANT N : natural := 2**K; -- FRAME AVERAGED2^K

   COMPONENT fifo
		GENERIC( N : natural; LEN : natural);
		PORT(clock : in std_logic;
		     reset : in std_logic;
			  enable : in std_logic;
			  input: in std_logic_vector(LEN-1 downto 0); 
			  output: out std_logic_vector(LEN-1 downto 0));
	END COMPONENT;
	
	COMPONENT accumulate
		GENERIC( LEN : natural);
		PORT(clock : in std_logic;
			  reset : in std_logic;
			  enable : in std_logic;
			  input: in std_logic_vector(LEN-1 downto 0); 
			  output: out std_logic_vector(LEN-1 downto 0));
	END COMPONENT;	


   COMPONENT clock_generator
      PORT( CLOCK2_50 : IN STD_LOGIC;
            reset    : IN STD_LOGIC;
            AUD_XCK  : OUT STD_LOGIC);
   END COMPONENT;

   COMPONENT audio_and_video_config
      PORT( CLOCK_50, reset : IN    STD_LOGIC;
            I2C_SDAT        : INOUT STD_LOGIC;
            I2C_SCLK        : OUT   STD_LOGIC);
   END COMPONENT;   

   COMPONENT audio_codec
      PORT( CLOCK_50, reset, read_s, write_s               : IN  STD_LOGIC;
            writedata_left, writedata_right                : IN  STD_LOGIC_VECTOR(23 DOWNTO 0);
            AUD_ADCDAT, AUD_BCLK, AUD_ADCLRCK, AUD_DACLRCK : IN  STD_LOGIC;
            read_ready, write_ready                        : OUT STD_LOGIC;
            readdata_left, readdata_right                  : OUT STD_LOGIC_VECTOR(23 DOWNTO 0);
            AUD_DACDAT                                     : OUT STD_LOGIC);
   END COMPONENT;
	
   SIGNAL read_ready, write_ready, read_s, write_s : STD_LOGIC;
   SIGNAL readdata_left, readdata_right            : STD_LOGIC_VECTOR(23 DOWNTO 0);
   SIGNAL writedata_left, writedata_right          : STD_LOGIC_VECTOR(23 DOWNTO 0);   
   SIGNAL reset                                    : STD_LOGIC;
	SIGNAL noise												: STD_LOGIC_VECTOR(23 DOWNTO 0);
	SIGNAL x0L, x0R, xS0L, xS0R, xSNL, xSNR     		: STD_LOGIC_VECTOR(23 DOWNTO 0);
	SIGNAL xL, xR, yL, yR       							: STD_LOGIC_VECTOR(23 DOWNTO 0);
	SIGNAL tNL, tNR											: STD_LOGIC_VECTOR(23 DOWNTO K);
	SIGNAL OutData_left, OutData_right					: STD_LOGIC_VECTOR(23 DOWNTO 0);
 
BEGIN
   reset <= NOT(KEY(0));

   --YOUR CODE GOES HERE
   --writedata_left <= readdata_left;
   --writedata_right <= readdata_right;
   read_s <= read_ready and write_ready;
   write_s <= read_ready and write_ready;
	
	
	x0L <= readdata_left+noise;
	x0R <= readdata_right+noise;
	
fifoL: fifo 
	GENERIC MAP(N, 24-K)
	PORT MAP(CLOCK_50, reset, read_s, x0L(23 DOWNTO k), tNL);
	
	
shiftL:	
	xSNL(23 downto 23-K+1) <= (others=> tNL(23));
	xSNL(23-K downto 0) <= tNL;
	xS0L(23 downto 23-K+1) <= (others=> x0L(23));
	xS0L(23-K downto 0) <= x0L(23 downto K);
	
	xL <= xSNL-xS0L;
	
accL: accumulate
	GENERIC MAP(24)
	PORT MAP(CLOCK_50, reset, read_s, xL, yL);

fifoR: fifo 
	GENERIC MAP(N, 24-K)
	PORT MAP(CLOCK_50, reset, read_s, x0R(23 DOWNTO k), tNR);
	
	
shiftR:	
	xSNR(23 downto 23-K+1) <= (others=> tNR(23));
	xSNR(23-K downto 0) <= tNR;
	xS0R(23 downto 23-K+1) <= (others=> x0R(23));
	xS0R(23-K downto 0) <= x0R(23 downto K);
	
	xR <= xSNR-xS0R;
	
accR: accumulate
	GENERIC MAP(24)
	PORT MAP(CLOCK_50, reset, read_s, xR, yR);	
	
	

NG: entity work.noise_generator(Behavior)
    port map( CLOCK_50, read_s, noise);	
   
   my_clock_gen: clock_generator PORT MAP (CLOCK2_50, reset, AUD_XCK);
   cfg: audio_and_video_config PORT MAP (CLOCK_50, reset, FPGA_I2C_SDAT, FPGA_I2C_SCLK);
   codec: audio_codec PORT MAP (CLOCK_50, reset, read_s, write_s, writedata_left, 
	                             writedata_right, AUD_ADCDAT, AUD_BCLK, AUD_ADCLRCK,
										  AUD_DACLRCK, read_ready, write_ready, readdata_left, 
										  readdata_right, AUD_DACDAT);
										  
	writedata_left <= yL when SW="00" else 
							x0L when SW="01" else
							readdata_left when SW = "10" else
							noise;
	writedata_right<= yR when SW="00" else 
							x0R when SW="01" else
							readdata_right when SW = "10" else
							noise;
END Behavior;


	
